package trader.gui;

import javax.swing.*;
import java.awt.*;
import trader.*;
import java.io.*;

public class StockGui implements Serializable {

  private JFrame frame;
  private Container contentPane;	
  private JPanel stockPan = new JPanel();
  private JTextArea allStockTa = new JTextArea();
	
  // MAX_SYMBOL_SIZE taken from size of field declared in database
  private int MAX_SYMBOL_SIZE = 15;
  private int MAX_PRICE_SIZE = 10;

  private transient boolean repositioned = true;
	
  public StockGui (Stock [] allStocks) {
    System.out.println("StockGui.constructor: instance created");
    buildDisplay(allStocks);
  }
	
  private void buildDisplay(Stock [] allStocks){
    System.out.println("StockGui.buildDisplay");
		
    populateAllStocksDisplay(allStocks);
		
    allStockTa.setEditable(false);
		
    frame = new JFrame("BrokerTool Stock Page");
		
    contentPane = frame.getContentPane();
    contentPane.setLayout(new BorderLayout());
		
    JScrollPane scroll = new JScrollPane(allStockTa);
    contentPane.add(scroll, BorderLayout.CENTER);

    frame.setLocation(new Point(300, 0));
		
    frame.pack();
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setVisible( true );
  }
	
  // build all stocks text area
  private void populateAllStocksDisplay(Stock [] allStocks) {
    System.out.println("StockGui.populateAllStocksTa:  writing stocks to textarea"); 
		
    // make font a fixed width for nicer appearance
    allStockTa.setFont(new Font("courier", Font.PLAIN, 12));
    for (int i = 0; i < allStocks.length; i++) {
			
      // do not need to append spaces to symbol because the
      // symbol size will be the size declared in database
      String symbol = allStocks[i].getSymbol();
      float price = allStocks[i].getPrice();
			
      // The length of the stock prices vary so need to append
      // the appropriate number of spaces at the end of the price
      // to make them all size 10
      StringBuffer stockPrice = new StringBuffer(Float.toString(price));
			
      for (int j = stockPrice.length(); j < MAX_PRICE_SIZE; j++) {
        stockPrice.append(" ");
      }
			
      // write symbol, price, and carriage return to textarea
      allStockTa.append(symbol);
      allStockTa.append(stockPrice.toString());
      allStockTa.append("\n");
    }    
  }

  public void updateStockDisplay (Stock stock) throws BrokerException {
		
    System.out.println("StockGui.updateStockDisplay: changing display for "
      + stock.getSymbol());
		
    // test to see if frame has been serialized
    if (!repositioned) {
      frame.setLocation(new Point(0, 0));
      repositioned = true;
      frame.setVisible( true );
    }

    // get the name of the stock to be updated
    String symbol = stock.getSymbol();
		
    // get the new price of the stock
    float price = stock.getPrice();
		
    // retrieve all the text from the textarea
    String allStockString = allStockTa.getText();
		
    // get the beginning position of the stock symbol to be updated
    int stockPosition = allStockString.indexOf(symbol);
		
    // if stock symbol is not found then throw an exception
    if (stockPosition == -1) {
      throw new BrokerException("Symbol " + symbol + " not found");
    }
    // modify textarea to display new price for stock
    else {
			
      // modify new price size to MAX_PRICE_SIZE to make sure
      // that all values in the old price get overwritten
      StringBuffer newPrice = new StringBuffer(Float.toString(price));
			
      for (int i = newPrice.length(); i < MAX_PRICE_SIZE; i++) {
        newPrice.append(" ");
      }
			
      // calculate where new price starts and ends
      int start = stockPosition + MAX_SYMBOL_SIZE;
      int end = start + MAX_PRICE_SIZE;
			
      // replace old price with new price in the textarea
      allStockTa.replaceRange(newPrice.toString(), start, end);
    }
  }
}



