package trader.nw;

import trader.*;
import java.util.*;

public class StockModelNwImpl implements StockModel {
  
  ArrayList changeListeners = new ArrayList(10);
  NwClient nwClient;

  /** Creates new StockModelNwImpl */
  public StockModelNwImpl(NwClient nwClient) {
    System.out.println("StockModelNwImpl.constructor:  instance created");
    this.nwClient = nwClient;
  }

  // Stock model state change listener registration methods
  
  /**-----------------------------------------------------------
   * Adds requester to the list of objects to be notified when a
   * Stock object in the stock model alters state   
   */
  public void addChangeListener(StockView sv) throws BrokerException {
	    
    System.out.println("StockModelNwImpl.addChangeListener");

    StockCommand cmd;
    Object result;
    try {
      cmd = new AddChangeListenerCommand(sv);
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result = cmd.getResult();
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
  }

  // Stock segment state change methods
  
  /**-------------------------------------------------------------
   * Updates the stock price in the stock model
   */
  public void updateStockPrice(Stock stock) throws BrokerException {

    System.out.println("StockModelNwImpl.updateStockPrice: " + stock.getSymbol()
      + ", " + stock.getPrice());
    
    StockCommand cmd;
    Object result;
    try {
      // create new UpdateCustomerCommand
      cmd = new UpdateStockPriceCommand(stock);
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result = cmd.getResult();
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
  }
  
  // Stock segment state query methods
  
  /**-------------------------------------------------------------
   * Given symbol, return the Stock from the model
   */
  public Stock getStock(String symbol) throws BrokerException {
    
    System.out.println("StockModelNwImpl.getStock: " + symbol);

    StockCommand cmd;
    Object result;
    Stock stock = null;
    try {
    
      cmd = new GetStockCommand(symbol);
      
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result =  cmd.getResult();
      
      stock = (Stock) result;
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
    return stock;
  }

  /**-------------------------------------------------------------
   * Returns all stocks in the stock model
   */
  public Stock[] getAllStocks() throws BrokerException{

    System.out.println("StockModelNwImpl.getAllStocks");
    
    StockCommand cmd;
    Object result;
    Stock[] allStocks = null;
    try {
    
      cmd = new GetAllStocksCommand();
      
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result =  cmd.getResult();
      
      allStocks = (Stock[]) result;
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
    return allStocks;
  }
}

