package trader.db;

import java.util.*;
import java.sql.*;
import trader.*;

public class StockModelDbImpl implements StockModel {
  
  private ArrayList changeListeners = new ArrayList(10);
  private Connection theConnection;
	
  /** -----------------------------------------------------------
   *  No-argument constructor.
   *  This is needed because the parent class' constructor
   *  throws an exception
   */
  public StockModelDbImpl() throws Exception {
    this("localhost");
  }
	
  public StockModelDbImpl(String hostName) throws Exception {
    String  database = "sample";
    String url;
    
    System.out.println("StockModelDbImpl.constructor: instance created");
    
    try {
      Class.forName("com.pointbase.jdbc.jdbcUniversalDriver"); 
      url = "jdbc:pointbase://" + hostName +
        "/" + database;
      theConnection = DriverManager.getConnection
        (url, "public", "public");
    } catch (Exception e) {
      System.out.println("StockModelDbImpl.constructor: " + e);
      throw e;
    }
  }
  
   /** ---------------------------------------------------------
   * This is a helper method used by many methods.
   * It returns a Connection object. 
   * In a single threaded environment you can return a single
   * connection object stored in the attribute theConnection.
   * In a multi-threaded environment each thread will require
   * its own connection. A future subclass of this class can
   * override this method and obtain the connection object 
   * from a connection pool.
   */
  private Connection obtainConnection() throws SQLException{
    //System.out.println("StockModelDbImpl.obtainConnection:  connected to db");
    
    // return the theConnection attribute
    return theConnection;
  }


  // Stock model state change listener registration methods
	
  /**-----------------------------------------------------------
   * Adds requester to the list of objects to be notified when a
   * Stock object alters state   
   */
  public void addChangeListener(StockView sv) throws BrokerException {
		
    System.out.println("StockModelDbImpl.addChangeListener: " +
      "change listener registered");
		
    // add sv to list of changeListeners
    changeListeners.add(sv);
  }
	
	
  /** -----------------------------------------------------------
   * This method notifies all registered StockView listeners
   * that a stock object has changed.
   */
  private void fireModelChangeEvent(Stock stock) {
    StockView sv;
		
    System.out.println("StockModelDbImpl.fireModelChangeEvent ("+
      stock.getSymbol() + ", " + stock.getPrice() + ")" ); 
	
    for (int i=0; i<changeListeners.size(); i++) {
      try {
				
	// get listener from array list
        sv = (StockView)changeListeners.get(i);
	sv.handleStockChange(stock);
			
      } catch(Exception e) {
	System.out.println("StockModelDbImpl.fireModelChangeEvent: " + e);
      }
    }
  }
	
  // Stock model state change methods
	
  /** -----------------------------------------------------------
   * changes price for the given stock
   */
  public void updateStockPrice(Stock stock) throws BrokerException {
		
    //System.out.println("StockModelDbImpl.updateStockPrice");
		
    try {
      float price;
      String symbol;
			
      Connection con;
      Statement stmt;
      ResultSet result;
      int rowCount;
			
      price = stock.getPrice();
      symbol = stock.getSymbol();
			
      /*  get connection to db and look up stock symbol  */
      con = obtainConnection();
      stmt = con.createStatement();
      String request = "SELECT * from Stock where symbol=" + "'"
        + symbol + "'";
			
      result = stmt.executeQuery(request);
			
      /* if symbol is valid then update stock price and notify
	       listeners of stock change
	       else throw an exception indicating invalid stock symbol
       */
      if (result.next()) {
			
        request = "UPDATE Stock set price=" + price
          + " where symbol=" + "'" + symbol + "'";
				
        System.out.println("StockModelDbImpl.updateStockPrice: " + request);
			
        rowCount = stmt.executeUpdate(request);
			
        // notify listeners that stock has changed
        fireModelChangeEvent(stock);
      }
      else {
        throw new BrokerException("Stock symbol " + symbol 
          + " not found");
      }
			
    } catch (Exception e) {
      System.out.println("StockModelDbImpl.updateStockPrice: " + e);
      throw new BrokerException("StockModelDbImpl.updateStockPrice" + e);
    }
  }
  
  // Stock model state query methods  
  
  /** -----------------------------------------------------------
   * gets Stock corresponding to the symbol
   */
  public Stock getStock(String sym) throws BrokerException {
		
    System.out.println("StockModelDbImpl.getStock");
		
    try {
      Stock stock = null;
      Connection con;
      Statement stmt;
      ResultSet result;
			
      /*  get connection to db and look up stock symbol  */
      con = obtainConnection();
      stmt = con.createStatement();
      String request = "SELECT price from Stock where symbol=" + "'" 
        + sym + "'";
	  
      System.out.println("StockModelDbImpl.getStock:  " + request);
	  	
      result = stmt.executeQuery(request);
			
      /* If symbol is found in database, get price else
         throw exception to indicate stock symbol not found
       */
      if (result.next()) {
        float price = result.getFloat(1);
        stock = new Stock(sym, price);
        return stock;
      } 
      else {
        throw new BrokerException("Record for stock " + sym 
          + " not found");
      }
	  
    } catch (Exception e) {
      System.out.println("StockModelDbImpl.getStock: " + e);
      throw new BrokerException("StockModelDbImpl.getStock: " + e);
    }
		
  }
	
	
  /** -----------------------------------------------------------
   * returns all stock in the enterprise data store
   */
  public Stock[] getAllStocks() throws BrokerException {
		
    System.out.println("StockModelDbImpl.getAllStocks");
		
    try {
      Stock[] allStocks = null;
      ArrayList stockList = new ArrayList(1);
      Stock[] temp = new Stock[1];
      Stock stock = null;
      Connection con;
      Statement stmt;
      ResultSet result;
			
      con = obtainConnection();
      stmt = con.createStatement();
      String request = "SELECT * FROM STOCK";
			
      System.out.println("StockModelDbImpl.getAllStocks:  " + request);
	  	
      result = stmt.executeQuery(request);
			
      while (result.next()) {
        String symbol = result.getString(1);
	float price = result.getFloat(2);
	stock = new Stock(symbol, price);
	stockList.add(stock);
      } 
			
      if (stockList.size() > 0) {
        allStocks = (Stock[]) stockList.toArray(temp);
      }
      else {
        allStocks = null;
      }
      return allStocks;
	  
    } catch (Exception e) {
      System.out.println("StockModelDbImpl.getAllStocks: " + e);
      throw new BrokerException("StockModelDbImpl.getAllStocks" + e);
    }
		
  }
  
}



