package trader;

import java.text.DecimalFormat;

public class StockMarket implements Runnable {

  private StockModel model;
  private Thread thread;
	
  public StockMarket (StockModel model) {
    System.out.println("\nStockMarket.constructor: instance created");
    this.model = model;
    thread = new Thread(this);
    thread.start();
  }
    
  public void run() {

    try {
      // short sleep to let other objects initialize first
      Thread.sleep(10000); 
    } catch (Exception e) {
      System.out.println(e.toString());
    }
    
    System.out.println("StockMarket.run:  initializing stock market");
    
    try{	
      // retrieve list of all stocks from model
      Stock [] allStocks = model.getAllStocks();
		
      // retrieve number of stocks
      int numStocks = allStocks.length;
		
      int randomStock;
      float randomPrice;
      Stock test;
      boolean rising = true;
		
      // This while loop uses a simple algorithm to simulate
      // the rising and falling of stock prices
      // Every 10 seconds a random stock will be chosen
      // to be updated.  Also a random price of less than a dollar
      // will be selected to modify the existing stock price by
      // Each time through the loop it will alternate adding
      // the random price to the current price and subtracting the
      // random price from the current price.
		
      System.out.println("StockMarket.run: initialization completed");
      
      while(true) {
			
        // sleep for 10 seconds
        Thread.sleep(10000);
			
        // get random number to select stock to update
        randomStock = (int)(Math.random() * numStocks);
			
        // get random number to calculate price changed
        randomPrice = (float)(Math.random() * 1.00);
			
        // format price to have 3 decimal places
        // Note that once the price is formatted it gets returned
        // as a String
			
        // format the amount to be added or subtracted to contain
        // 2 decimal places
        DecimalFormat randPriceFormat = new DecimalFormat("#.##");
			
        // format the final price of a stock to have 3 decimal
        // places.  3 decimal places was chosen because in the
        // existing database there are some stocks that already
        // have a 3 decimal place price
        DecimalFormat finalPriceFormat = new DecimalFormat("#.###");
			
        // format the amount to be added or subtracted to the current price
        // Note that the formatted value is of type String
        String formattedRandomPrice = randPriceFormat.format(randomPrice);
			
        // retrieve the stock to be updated
        test = new Stock(allStocks[randomStock].getSymbol(), 
        allStocks[randomStock].getPrice());
			
        // retrieve the current price of the stock
        float price = test.getPrice();
			
        // converted formatted value to be added or subtracted
        // from the current price back to a float
        randomPrice = Float.parseFloat(formattedRandomPrice);
			
        // If boolean value rising is set to true then add
        // amount to stock price else subtract amount
        if (rising) {
          price = price + randomPrice;
          rising = false;
        }
        else {
          price = price - randomPrice;
          rising = true;
        }
			
        // format the final price of the stock to contain
        // 3 decimal places
        // This is done to eliminate strange floating point
        // arithmetic results.  For example 24.62 + .18 YIELDS 24.800001
        // ??? NEED BETTER WORDING HERE FOR COMMENT ??????
        String formattedPrice = finalPriceFormat.format(price);
				
        // set new price of stock 
        test.setPrice(Float.parseFloat(formattedPrice));
			
        // update the model with new stock price
        System.out.println("\nStockMarket.run: updating stock price " + test);
        model.updateStockPrice(test);		
      }  
    } catch (Exception e) {
      System.out.println(e.toString());
    }
  }
}



