package trader.nw;

import trader.*;
import java.util.*;
import java.net.*;

public class StockModelNwImpl2 implements StockModel {
  
  ArrayList changeListeners = new ArrayList(10);
  NwClient nwClient;

  // new attributes for StockModelNwImpl2
  StockView stockView;
  StockViewNwImpl stockViewNw;
  NwServer nwServer;
  StockViewCommandServer stockViewCmdServer;
  int nwServerViewPortNo = 7010;
	
  /** Creates new StockModelNwImpl2 */
  public StockModelNwImpl2(NwClient nwClient) {
    System.out.println("StockModelNwImpl2.constructor:  instance created");
    this.nwClient = nwClient;
  }

  // Stock model state change listener registration methods
  
  /**-----------------------------------------------------------
   * Adds requester to the list of objects to be notified when a
   * Stock object in the stock model alters state   
   */
  public void addChangeListener(StockView sv) throws BrokerException {
	    
    System.out.println("StockModelNwImpl2.addChangeListener");

    StockCommand cmd;
    Object result;
    
    try {
      stockView = sv;
      stockViewCmdServer = new StockViewCommandServer(sv);
      
      stockViewNw = new StockViewNwImpl("localhost", nwServerViewPortNo);
      cmd = new AddChangeListenerCommand(stockViewNw);
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result = cmd.getResult();
      nwServer = new NwServer(nwServerViewPortNo, stockViewCmdServer);
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
  }

  // Stock segment state change methods
  
  /**-------------------------------------------------------------
   * Updates the stock price in the stock model
   */
  public void updateStockPrice(Stock stock) throws BrokerException {

    System.out.println("StockModelNwImpl2.updateStockPrice: " + stock.getSymbol()
      + ", " + stock.getPrice());
    
    StockCommand cmd;
    Object result;
    try {
      // create new UpdateStockCommand
      cmd = new UpdateStockPriceCommand(stock);
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result = cmd.getResult();
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
  }
  
  // Stock segment state query methods
  /**-------------------------------------------------------------
   * Given a symbol, return the Stock from the model
   */
  public Stock getStock(String symbol) throws BrokerException {
    
    System.out.println("StockModelNwImpl2.getStock: " + symbol);

    StockCommand cmd;
    Object result;
    Stock stock = null;
    try {
      cmd = new GetStockCommand(symbol);
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result =  cmd.getResult();
      
      stock = (Stock) result;
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
    return stock;
  }

  /**-------------------------------------------------------------
   * Returns all stocks in the stock model
   */
  public Stock[] getAllStocks() throws BrokerException{

    System.out.println("StockModelNwImpl2.getAllStocks");
    
    StockCommand cmd;
    Object result;
    Stock[] allStocks = null;
    try {
    
      cmd = new GetAllStocksCommand();
      
      nwClient.send(cmd);
      cmd = (StockCommand) nwClient.receive();
      result =  cmd.getResult();
      
      allStocks = (Stock[]) result;
    } catch(Exception e) {
      throw new BrokerException(e.toString());
    }
    return allStocks;
  }
}


